# Section 4.4
# When sample is different, i.e., area 1-5 = 1, area 6-10 = 2, area 11-15 = 3 and area 16-20 =4

########################################################
# Results of proposed parametric and semi-parametric method
########################################################
# Version: July 11, 2018


## Required packages
library(VineCopula)
library(copula)
library(MASS)
library(sn)


## Caution: Load the "SAE_Func_Diff_ni.R" and "Sim_Func_Diff_ni.R" R scripts
source("SAE_Func_Diff_ni.R")
source("Sim_Func_Diff_ni.R")


#### Setting for the simulation set-up
m = 20  ## Number of small areas
n <- c(rep(1,5), rep(2,5), rep(3,5), rep(4,5))
N <- rep(200, m)

## draw X from normal distribution with mean and standard deviation = 1
#set.seed(123000)
data.X <- X <- matrix(c(rnorm(200*m, mean = 1, sd = 1)),m,200) #generate populaiton of X from N(1,1)


family = "frank"
beta <- c(1,1) ## Beta0 and Beta1
tau = 0.33

## 1.Normal Distribution
mar.dist = c("norm")
mar.par = c(0,1)

## Submit the AreaID required
areaID <- c(rep(1:5,1), rep(6:10,each = 2), rep(11:15,each = 3), rep(16:20,each = 4)) ## Change it as per your need


R = 500 ## Number of Simulation runs
B = 100 ## Number of bootstrap samples
output <- matrix(NA, ncol = 23)


##############################################################################
## Simulation Setup
##############################################################################

for (r in 1:(R)){
  
  set.seed(r)
  # colnames(output) <- c("par.p", "par.spar", "tau.par", "tau.spar", "beta0.hat", "beta1.hat", "mar.par1.hat", "mar.par2.hat", "bias.par", "bias.semi", "MSE.par", "MSE.spar", "MSE1.par", "MSE1.spar", "MSE2.par", "MSE2.spar","MSE3.par", "MSE3.spar", "bias.boot.par", "bias.boot.spar", "mse.boot.par", "mse.boot.spar")
  output2 = tryCatch(SA.simMC(n,N, X, family, tau, beta = c(1,1), mar.dist, mar.par,areaID, 100), error = function(e) {return(NA)})
  
  
  output <- rbind(output, output2)
  
  cat(r)
  if(r==500) break
}




out = data.frame(na.omit(output))
final.result <-  aggregate(out, list(out$group), mean)

# result = Return the output for average of areaID's with same number of sample sizes
# group = shows 4 groups with same number of sample size
# par.p = estimated copula parameter when it is parametric model
# par.spar = estimated copula parameter when it is semi-parametric model
# tau.par = tau value after coversion of par.p for a given copula family
# tau.spar = tau value after coversion of par.spar for a given copula family
# beta0.hat = estimated beta_0 value
# beta1.hat = estimated beta_1 value
# mar.par1.hat = estimated value of first marginal parameter for e.g., mean for normal distribution
# mar.par2.hat = estimated value of second marginal parameter for e.g., variance for normal distribution
# bias.par = response bias under parametric model
# bias.semi = response bias under semi-parametric model
# MSE.par = EMSPE under parametric model
# MSE.spar = EMSPE under semi-parametric model
# Similarly, MSE1.par = M1i under parametric model till MSE3.spar = M3i under semi-parametric model
# bias.boot.par = response bias under parametric while MSPE estimation using bootstrap method
# mse.boot.par = mean of MSPE of the EBUP while MSPE estimation under parametric model
# Similarly, for semi-parametric model use .spar instead .par for the above mentioned two comments 





