# Section 4.1
# When data is generated from clayton copula with error margins as normal distribution
# for the case when m=20. The model is correctly specified while doing estimation procedure.


########################################################
# Results of proposed parametric and semi-parametric method
########################################################
# Version: August 19, 2019


## Required packages
library(VineCopula)
library(copula)
library(MASS)
library(sn)
library(nlme)

## Caution: Load the "SAE_Funct.R" and "Sim_Output_Funct.R" R scripts 
source("SAE_Funct.R")
source("Sim_Output_Funct.R")


#### Setting for the simulation set-up
m = 20  ## Number of small areas
n <- rep(4,m)  ## Number of sample taken from each area
N <- rep(200, m)  ## Number of population units in each area

## draw X from normal distribution with mean and standard deviation = 1
#set.seed(123000)
data.X <- X <- matrix(c(rnorm(200*m, mean = 1, sd = 1)),m,200) #generate populaiton of X from N(1,1)


family = "clayton" ## Change it to "frank", "normal" and "gumbel" for others copula families
beta <- c(1,1) ## Beta0 and Beta1
tau = 0.33 ## Kendall's Tau

## Distributional assumption for error margins
## 1.Normal Distribution
mar.dist = c("norm")
mar.par = c(0,1)


## 2. Skewed Normal Distribution: For the case when gamma is 10 
# mar.dist = c("snorm")
# omega = 1.651731
# alpha = 10
# xi = -1.311350534
# mar.par = c(omega, alpha, xi)

R = 500 ## Number of Simulation runs
B = 100 ## Number of bootstrap samples
output <- data.frame(matrix(NA, nrow = R*m, ncol = 23)) ## Change ncol = 24 of it is skewed-normal distribution

##############################################################################
## Simulation Setup
##############################################################################

for (r in 1:(R)){
  
  set.seed(r)
  out = tryCatch(SA.simMC(n,N, X, family, tau, beta = c(1,1), mar.dist, mar.par, B), error = function(e) {return(NA)})
  
  if (r==1){
    output = out
  }
  
  else{
    output = rbind(output, out)
  }
  
  colnames(output) <- c("par.p", "par.spar", "tau.par", "tau.spar", "beta0.hat", "beta1.hat", "mar.par1.hat", "mar.par2.hat", "bias.par", "bias.semi", "MSE.par", "MSE.spar", "MSE1.par", "MSE1.spar", "MSE2.par", "MSE2.spar","MSE3.par", "MSE3.spar", "bias.boot.par", "bias.boot.spar", "mse.boot.par", "mse.boot.spar", "areaID")
  
  cat(r)
  
}


out = na.omit(output)
result = round(aggregate(out[, 1:22], list(out[,23]), mean),3)
colnames(result) <- c("areaID","par.p", "par.spar", "tau.par", "tau.spar", "beta0.hat", "beta1.hat", "mar.par1.hat", "mar.par2.hat", "bias.par", "bias.semi", "MSE.par", "MSE.spar", "MSE1.par", "MSE1.spar", "MSE2.par", "MSE2.spar","MSE3.par", "MSE3.spar", "bias.boot.par", "bias.boot.spar", "mse.boot.par", "mse.boot.spar")



# result = Return the output where
# par.p = estimated copula parameter when it is parametric model
# par.spar = estimated copula parameter when it is semi-parametric model
# tau.par = tau value after coversion of par.p for a given copula family
# tau.spar = tau value after coversion of par.spar for a given copula family
# beta0.hat = estimated beta_0 value
# beta1.hat = estimated beta_1 value
# mar.par1.hat = estimated value of first marginal parameter for e.g., mean for normal distribution
# mar.par2.hat = estimated value of second marginal parameter for e.g., variance for normal distribution
# bias.par = response bias under parametric model
# bias.semi = response bias under semi-parametric model
# MSE.par = EMSPE under parametric model
# MSE.spar = EMSPE under semi-parametric model
# Similarly, MSE1.par = M1i under parametric model till MSE3.spar = M3i under semi-parametric model
# bias.boot.par = response bias under parametric while MSPE estimation using bootstrap method
# mse.boot.par = mean of MSPE of the EBUP while MSPE estimation under parametric model
# Similarly, for semi-parametric model use .spar instead .par for the above mentioned two comments 
# areaID = ID of m small areas







