##############################################################################
## Simulation Function for Monte-Carlo Study
##############################################################################

SA.simMC = function(n, N, X.mat, family,misfamily, tau, beta, mar.dist, mar.par, B){
  
  # Obtain Copula Parameter:
  
  if(misfamily=="normal"){misfamnum = 1}
  if(misfamily=="t"){misfamnum = 2}     
  if(misfamily=="clayton"){misfamnum = 3}
  if(misfamily=="gumbel"){misfamnum = 4}
  if(misfamily=="frank"){misfamnum = 5}
  
  
  par = VineCopula::BiCopTau2Par(family=misfamnum, tau=tau)
  
  
  # Generate Population:
  cop.data = SA.rCopula(N=N, family=misfamily, par=par)
  
  U = cop.data$U
  AreaID = cop.data$AreaID  
  Error = SA.U2Error(U, mar.dist = mar.dist, mar.par=mar.par)
  
  Pop.data <- SA.rPop(N=N, AreaID=AreaID, Error=Error, X.mat=X.mat, beta = beta)
  Y.mat <- Pop.data$Y.mat                         
  
  Y.bar <- Pop.data$Y.bar 
  X.bar <- apply(X.mat,1,mean)
  
  
  # Obtain Sample:
  
  Sample.data <- SA.rSample(n=n, Y.mat, X.mat)
  y <- Sample.data$y                        
  x <- Sample.data$x 
  error <-  y - (beta[1] + x*beta[2])    # true error
  u <- SA.Error2U(error, mar.dist = mar.dist, mar.par=mar.par)
  
  areaID <- rep(1:m, each=n[1])  # Caution: be careful if sample sizes are different.
  
  
  
  # Benchmark: BUP under the true model:
  
  e.hat.true <- SA.pred.error(error=error, u=u, areaID=areaID, family=misfamily, par=par)  # true copula 
  e.bar.true <- aggregate(error, by = list(areaID), FUN = "mean")$x 
  Yi_BUP <-  SA.pred.mean(N=N, n=n, e.hat = e.hat.true, e.bar=e.bar.true, beta = beta, X.bar=X.bar) # true beta
  
  
  
  # Approach 1: Parametric Estimation
  
  results.par <- SA.Est.par(y=y, x=x, areaID=areaID, family=family, mar.dist=mar.dist) 
  e.hat.par   <- SA.pred.error(error=results.par$resids, u=results.par$u, areaID=areaID, 
                               family=family, par=results.par$par) 
  Yi_EBUP.par <-SA.pred.mean(N=N, n=n, e.hat = e.hat.par, e.bar=results.par$residbar, 
                             beta = results.par$beta, X.bar=X.bar)
  
  
  
  # Approach 2: Semi-parametric Estimation
  
  results.spar <- SA.Est.spar(y=y, x=x, areaID=areaID, family=family, est.method= "mle") 
  e.hat.spar <- SA.pred.error(error=results.spar$resids, u=results.spar$u, areaID=areaID, 
                              family=family, par=results.spar$par) 
  Yi_EBUP.spar <-SA.pred.mean(N=N, n=n, e.hat = e.hat.spar, e.bar=results.spar$residbar, 
                              beta = results.spar$beta, X.bar=X.bar)
  

  ## 1. Fully parametric approach 
  ##  bias and mse of EBUP -- averaged over m small areas
  Bias.par <- ( Yi_EBUP.par - Y.bar)
  MSE.par  <- ((Yi_EBUP.par - Y.bar)^2)
  MSE1.par <- ((Yi_BUP      - Y.bar)^2)
  MSE2.par <- ((Yi_EBUP.par - Yi_BUP )^2)
  MSE3.par <- (2*(Yi_BUP - Y.bar)*(Yi_EBUP.par - Yi_BUP))
  
  
  ## 2.Semi-parametric approach
  ##  bias and mse of EBUP -- averaged over m small areas
  Bias.spar <- (Yi_EBUP.spar - Y.bar)
  MSE.spar  <- ((Yi_EBUP.spar - Y.bar)^2)
  MSE1.spar <- ((Yi_BUP      - Y.bar)^2)
  MSE2.spar <- ((Yi_EBUP.spar - Yi_BUP )^2)
  MSE3.spar <- (2*(Yi_BUP - Y.bar)*(Yi_EBUP.spar - Yi_BUP))
  
  
  
  ### BOOTSTRAP for MSE estimation:
  for(bb in 1:B){
    
    boot.p <- SA.simBoot.par(n=n, N=N, X.mat = X.mat, family=family, par= results.par$par, 
                             beta= results.par$beta, mar.dist=mar.dist, mar.par=results.par$mar.par)
    
    if(bb==1){
      boot.par = boot.p
    }
    else{
      boot.par = rbind(boot.par, boot.p)
    }
    
    
    
    boot.sp <- SA.simBoot.spar(n=n, N=N, X.mat = X.mat, family=family, par= results.spar$par, 
                               beta= results.spar$beta, obs.error = results.spar$resids, 
                               est.method="mle")
    if(bb==1){
      boot.spar = boot.sp
    }
    else{
      boot.spar = rbind(boot.spar, boot.sp)
    }
    
  }
  
  mean.boot.par <- aggregate(boot.par[, 2:3], list(boot.par[,1]), mean)
  mean.boot.spar <- aggregate(boot.spar[, 2:3], list(boot.spar[,1]), mean)
  colnames(mean.boot.par) <- c("areaID", "Bias.boot.par", "MSE.boot.par")
  colnames(mean.boot.spar) <- c("areaID", "Bias.boot.spar", "MSE.boot.spar")
 
  
  if(family=="normal"){famnum = 1}
  if(family=="t"){famnum = 2}     # Caution: be careful with the second parameter of t copula.
  if(family=="clayton"){famnum = 3}
  if(family=="gumbel"){famnum = 4}
  if(family=="frank"){famnum = 5}
  
  par.hat  = matrix(c(rep(results.par$par, times=m), rep(results.spar$par, times=m)), ncol=2)
  tau.hat  = VineCopula::BiCopPar2Tau(family=famnum, par = par.hat)
  beta.hat = matrix(c(rep(results.par$beta[1], times=m), rep(results.par$beta[2], times=m)), ncol=2)        
  mpar.hat = matrix(c(rep(results.par$mar.par[1], times=m), rep(results.par$mar.par[2], times=m)), ncol=2)   
  
  Bias = t(rbind(Bias.par, Bias.spar))
  MSE = t(rbind(MSE.par, MSE.spar))
  MSE1 = t(rbind(MSE1.par, MSE1.spar))
  MSE2 = t(rbind(MSE2.par, MSE2.spar))
  MSE3 = t(rbind(MSE3.par, MSE3.spar))
  
  Bias.boot = t(rbind(mean.boot.par[,2], mean.boot.spar[,2]))
  MSE.boot = t(rbind(mean.boot.par[,3], mean.boot.spar[,3]))
  
  result = matrix(c(par = par.hat, tau = tau.hat, beta = beta.hat, mar.par= mpar.hat, 
                    Bias = Bias, MSE=MSE, MSE1= MSE1, MSE2=MSE2, MSE3= MSE3, 
                    Bias.boot=Bias.boot, MSE.boot=MSE.boot, unique(areaID)),nrow = m, ncol=23)
  return(result)
  
}



