# Section 4.2
# When data is generated from Clayton copula with normal marginal distribution
# for the case when m=20. But for the estimation procedure it is misspecified as Gaussian copula.


########################################################
# Results of proposed parametric and semi-parametric method
########################################################
# Version: August 19, 2019

## Required packages
library(VineCopula)
library(copula)
library(MASS)
library(sn)


## Caution: Load the "SAE_Funct_Cop_Misspecified.R" and "SAE_Funct_Cop_Misspecified.R" R scripts
source("SAE_Funct_Cop_Misspecified.R")
source("SAE_Funct_Cop_Misspecified.R")


#### Setting for the simulation set-up
m = 20  ## Number of small areas
n <- rep(4,m)  ## Number of sample taken from each area
N <- rep(200, m)  ## Number of population units in each area


## draw X from normal distribution with mean and standard deviation = 1
#set.seed(123000)
data.X <- X <- matrix(c(rnorm(200*m, mean = 1, sd = 1)),m,200) # generate populaiton of X from N(1,1)

# Misspecified copula
misfamily = "clayton" ## Population generated from Gaussian copula


family = "normal"  ## Copula family used in the estimation procedure. Change it to "gumbel" or "frank" for other results in tables 
beta <- c(1,1) ## Beta0 and Beta1
tau = 0.33

## 1.Normal Distribution
mar.dist = c("norm")
mar.par = c(0,1)

R = 500 ## Number of Simulation runs
B = 100 ## Number of bootstrap samples
output <- data.frame(matrix(NA, nrow = R*m, ncol = 23)) ## Change ncol = 24 of it is skewed-normal distribution

##############################################################################
## Simulation Setup
##############################################################################

for (r in 1:(R)){
  
  set.seed(r)
  out = tryCatch(SA.simMC(n,N, X, family, misfamily = misfamily, tau, beta = c(1,1), mar.dist, mar.par, B), error = function(e) {return(NA)})
  
  if (r==1){
    output = out
  }
  
  else{
    output = rbind(output, out)
  }
  
  colnames(output) <- c("par.p", "par.spar", "tau.par", "tau.spar", "beta0.hat", "beta1.hat", "mar.par1.hat", "mar.par2.hat", "bias.par", "bias.semi", "MSE.par", "MSE.spar", "MSE1.par", "MSE1.spar", "MSE2.par", "MSE2.spar","MSE3.par", "MSE3.spar", "bias.boot.par", "bias.boot.spar", "mse.boot.par", "mse.boot.spar", "areaID")
  
  cat(r)
  
}


out = na.omit(output)
result = round(aggregate(out[, 1:22], list(out[,23]), mean),3)
colnames(result) <- c("areaID","par.p", "par.spar", "tau.par", "tau.spar", "beta0.hat", "beta1.hat", "mar.par1.hat", "mar.par2.hat", "bias.par", "bias.semi", "MSE.par", "MSE.spar", "MSE1.par", "MSE1.spar", "MSE2.par", "MSE2.spar","MSE3.par", "MSE3.spar", "bias.boot.par", "bias.boot.spar", "mse.boot.par", "mse.boot.spar")

# result = Return the output where
# par.p = estimated copula parameter when it is parametric model
# par.spar = estimated copula parameter when it is semi-parametric model
# tau.par = tau value after coversion of par.p for a given copula family
# tau.spar = tau value after coversion of par.spar for a given copula family
# beta0.hat = estimated beta_0 value
# beta1.hat = estimated beta_1 value
# mar.par1.hat = estimated value of first marginal parameter for e.g., mean for normal distribution
# mar.par2.hat = estimated value of second marginal parameter for e.g., variance for normal distribution
# bias.par = response bias under parametric model
# bias.semi = response bias under semi-parametric model
# MSE.par = EMSPE under parametric model
# MSE.spar = EMSPE under semi-parametric model
# Similarly, MSE1.par = M1i under parametric model till MSE3.spar = M3i under semi-parametric model
# areaID = ID of m small areas



